using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class AtrEnvelope : Indicator
    {
    	private ATR atr = null;
    
        public AtrEnvelope() : this(14, 0)
        { }

        public AtrEnvelope(int period, int barsAgo) : base()
        {   
        	this.BarsAgo = barsAgo;
            this.Period = period;
            base.PanelCount = 4;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                
                	AddPlot(new Plot("Up", new LinePlotStyle(), Brushes.Green, 2));
                	AddPlot(new Plot("Down", new LinePlotStyle(), Brushes.Brown, 2));
                
                    break;
                case State.StartUp:
                	
                	this.atr = AddOrGetIndicator.ATR(this, this.Period);
                
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
        	if (CurrentBar < this.BarsAgo)
        		return;
        		
            Values[0][0] = Input[this.BarsAgo] + this.atr[0];
            Values[1][0] = Input[this.BarsAgo] - this.atr[0];
        }
        
        private int barsAgo = 0;
        [Input]
        public int BarsAgo
        {
        	get { return barsAgo; }
        	set 
        	{
        		barsAgo = Math.Max(0, value);
        	}
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }
        
        public ISeries<double> Up
        {
        	get { return Values[0]; }
        }
        
        public ISeries<double> Down
        {
        	get { return Values[1]; }
        }

    }
    
    public static partial class AddOrGetIndicator
    {
    	public static AtrEnvelope AtrEnvelope(SharpScriptBase owner, int period, int barsAgo)
    	{
    		return AtrEnvelope(owner, owner.Input, period, barsAgo);
    	}
    	
    	public static AtrEnvelope AtrEnvelope(SharpScriptBase owner, ISeries<double> input, int period, int barsAgo)
    	{
    		return CacheIndicator<AtrEnvelope>(owner, input, new object[]{ period, barsAgo });
    	}
    
    
    }
    
}